#include <algorithm>
#include <numeric>
#include <iostream>
#include <limits>
#include <random>
#include <vector>

#include "utils.h"
#include "optimizer.h"
#include "coco.h"

using namespace std;
using Eigen::all;
using Eigen::seq;
using Eigen::seqN;
using Eigen::SelfAdjointEigenSolver;
using Eigen::Array;
using Eigen::Dynamic;

const double pi=3.14159265358979323846;

extern std::mt19937 generator;

ArrayXd Optimizer::getResDump(){
    return resultsDumps;
}

Optimizer::Optimizer( int dim, int funNmbr, int maxFES, double fitAtOpt, double fitError, int popSize, double initial_CR, double initial_F,
    std::vector<size_t> evalsToDump, double normSigma, double cauchySigma, double algProbResetThreshold, int initialSuccessCount, 
    int min_pop_size, int itToEnableStage2ForIDEDivider, double itOfChangeSRTDivider, double partOfPopUsedInEigenVect,
    double pEigCross, double smallCauchyThreshold, double arch_size_mult, int memSize, double maxPerOfUsedTheBest,
    double powShape, double minSuperiorRatio, double pertProbMult, double SRT, double firstCRThreshold,
    double secondCRThreshold, double firstCRMax, double secondCRMax, double tanEps, double tanWeight,
    double FUpperBound, double boundFThreshold, double FwFirstMult, double FwSecMult, double FwThirdMult, 
    double FwFirstThreshold, double FwSecondThreshold):
    dim(dim), funNmbr(funNmbr), maxFES(maxFES), fitAtOpt(fitAtOpt), fitError(fitError), initPopSize(popSize), initialCR(initial_CR), initialF(initial_F),
    fitEvals(0), bestError(numeric_limits<double>::infinity()), bestSol(dim), evalsToDump(evalsToDump), normSigma(normSigma),
    cauchySigma(cauchySigma), whenFound(maxFES), algProbResetThreshold(algProbResetThreshold), initialSuccessCount(initialSuccessCount),
    minPopSize(min_pop_size), itToEnableStage2ForIDEDivider(itToEnableStage2ForIDEDivider), itOfChangeSRTDivider(itOfChangeSRTDivider),
    partOfPopUsedInEigenVect(partOfPopUsedInEigenVect), pEigCross(pEigCross), smallCauchyThreshold(smallCauchyThreshold),
    archSizeMult(arch_size_mult), memSize(memSize), maxPerOfUsedTheBest(maxPerOfUsedTheBest), powShape(powShape),
    minSuperiorRatio(minSuperiorRatio), pertProbMult(pertProbMult), SRT(SRT), firstCRThreshold(firstCRThreshold),
    secondCRThreshold(secondCRThreshold), firstCRMax(firstCRMax), secondCRMax(secondCRMax), tanEps(tanEps), tanWeight(tanWeight),
    FUpperBound(FUpperBound),boundFThreshold(boundFThreshold), FwFirstMult(FwFirstMult), FwSecMult(FwSecMult),
    FwThirdMult(FwThirdMult), FwFirstThreshold(FwFirstThreshold), FwSecondThreshold(FwSecondThreshold){
}

Optimizer::~Optimizer(){
}

double Optimizer::callObjFunc(double* sol){
    double toRet;

    //cec22_test_func(sol, &toRet, dim, 1, funNmbr);
    coco_evaluate_function(problem, sol, &toRet);

    toRet -= fitAtOpt;
    ++fitEvals;
    if(toRet<bestErrorAtEv){
       bestErrorAtEv = toRet;
    }
    if(fitEvals==evalsToDump[curDumpIndx]){
        resultsDumps[curDumpIndx++] = bestErrorAtEv;
    }
    return toRet;
}

void Optimizer::saveInitialFitDump(){
    while(evalsToDump[curDumpIndx]<=fitEvals ){
        resultsDumps[curDumpIndx++] = bestErrorAtEv;
    }
}

double Optimizer::getBestError(){
    return bestError;
}

bool Optimizer::budgetLeft(){
    return fitEvals<maxFES;
}

ArrayXd Optimizer::popEval(const ArrayXXd& pop){
    ArrayXd fitToRet(popSize);
    for(int i=0; i<popSize; ++i){
        ArrayXd vectorXd = pop.row(i);
        fitToRet(i) = callObjFunc(vectorXd.data());
    }
    return fitToRet;
}

void Optimizer::rouleteAlgSel(int* successTab, int nmbrOfAlgs, int& selectedAlg, double& minProbOfAlgSel){
    double min = successTab[0]<successTab[1]?successTab[0]:successTab[1];
    double sum = successTab[0]+successTab[1];
    minProbOfAlgSel = min / sum;
    double probOfFirst = (double)successTab[0]/sum;
    if( probOfFirst>random(0,1) )
        selectedAlg = 1;
    else
        selectedAlg = 2;        
}

void Optimizer::optimize(){
    //coco_random_state_t *random_generator = coco_random_new(seed);
    coco_suite_t *suite;
    //coco_observer_t *observer;

    //char *observer_options;

    //char algorithm_id[200] = "onlyIDEcorrItMaxWithoutSortsNoIDEbdmutNoRandxr3CleaningWithoutFails";
    //observer_options = coco_strdupf("result_folder: %s "
    //               "algorithm_name: %s "
    //               "algorithm_info: \"Simplified EA4Eig\"", algorithm_id, algorithm_id);

    //char myOptions[] = "dimensions:5";  
    suite = coco_suite("bbob", "year: 2009", NULL);
    //observer = coco_observer("bbob", observer_options);
    problem = coco_suite_get_problem_by_function_dimension_instance(suite, funNmbr, dim, 1);

//cout<<problem->    
//void coco_evaluate_constraint(coco_problem_t *problem, const double *x, double *y);
/*    const double *lower = coco_problem_get_smallest_values_of_interest(problem);
    const double *upper = coco_problem_get_largest_values_of_interest(problem);
cout<<*lower<<*upper<<endl;*/
//cout<<problem->best_value[0]<<endl;
 //cout<<problem->final_target_delta[0]<<endl;
    fitAtOpt = coco_problem_get_best_value(problem);
//cout<<"fitAtOpt:"<<fitAtOpt<<endl;


    resultsDumps.setZero( evalsToDump.size() );
    curDumpIndx=0;
    bestErrorAtEv=numeric_limits<double>::infinity();
    const int L_BOUND=-5;
    const int UP_BOUND=5;
    popSize = initPopSize;
    //minPopSize = 10;
    if(minPopSize>initPopSize){
        minPopSize=initPopSize;
    }
    const int nmbrOfAlgs = 2;
    int successTab[nmbrOfAlgs];
    fill(successTab, successTab+nmbrOfAlgs, initialSuccessCount);
    pop = Eigen::ArrayXXd::NullaryExpr(popSize, dim,[&](){return random(L_BOUND, UP_BOUND);});
    fit.setConstant(popSize, numeric_limits<double>::infinity());

    fit = popEval(pop);
    saveInitialFitDump();
    auto it = min_element(fit.begin(), fit.end());
    bestError = *it;
    bestSol = pop.row(distance(fit.begin(), it));
    //ouble algProbResetThreshold = 1. / (5. * nmbrOfAlgs);

    //IDE params:
    int itMax = maxFES / popSize;
    int itToEnablestage2ForIDE = itMax/itToEnableStage2ForIDEDivider;
    int itOfChangeSRT = itMax / itOfChangeSRTDivider; // auxiliary count of generation for SRT threshold
    int itCounter4IDE = 0;
    int itsWithLowSRCounter = 0;

    //double partOfPopUsedInEigenVect = 0.5; //such part of P is selscetd fo eigenvestors...
    //double pEigCross = 0.4; //probability to use an eigenvector crossover

    ArrayXd CRTabUsedInEigen = ArrayXd::Zero(popSize);
    for(int i=0; i<popSize; ++i){
        if( random(0, 1) < smallCauchyThreshold ){
            CRTabUsedInEigen(i) = cauchyRand(0.1, cauchySigma);
        }else{
            CRTabUsedInEigen(i) = cauchyRand(0.95, cauchySigma);
        }
        if( CRTabUsedInEigen(i) > 1){
            CRTabUsedInEigen(i) = 1;
        }else if(CRTabUsedInEigen(i) < 0){
            CRTabUsedInEigen(i) = 0;
        }
    }

    // jSO params
    int maxArchSize = popSize * archSizeMult;
    //const int memSize = 5;
    double MF[memSize];
    fill(MF, MF+memSize, initialF);
    double MCR[memSize];
    fill(MCR, MCR+memSize, initialCR);
    MF[memSize-1] = 0.9;
    MCR[memSize-1] = 0.9;
    int curMemIndx = 0;
    int curArchSize = 0;
    ArrayXXd archive(maxArchSize, dim);
    //double maxPerOfUsedTheBest = 0.25;
    double minPerOfUsedTheBest = maxPerOfUsedTheBest / 2;

    double* FUsedInIt = new double[popSize];
    double* CRUsedInIt = new double[popSize];
    ArrayXd deltaFit;
    ArrayXi ord(popSize);
    ArrayXXd newPop(popSize, dim);
    //double SRT;
    ArrayXi selIndexes;
    ArrayXd xo, xr1, xr2, xr3;
    ArrayXd randIndiv;
    ArrayXd indAfterCross;
    ArrayXd mutant;
    VectorXd indEigAfterCross;
    VectorXd mutEig;
    ArrayXd newFit;
    Array<bool,Dynamic,1> indSucc;
    MatrixXd eigVect;

    //1 IDE; 2 jso
    while( (fitEvals < maxFES) && (bestError > fitError)){
        int selectedAlg;
        double minProbOfAlgSel;
        rouleteAlgSel(successTab, nmbrOfAlgs, selectedAlg, minProbOfAlgSel); // select algorithm to be used

        //reset ToDo - czy potrzebny
        if( minProbOfAlgSel < algProbResetThreshold ){
            for(int i=0; i<nmbrOfAlgs; ++i){
                successTab[i] = initialSuccessCount;
            }
        }  
//cout<<"selectedAlg:"<<selectedAlg<<" "<<100.0*fitEvals/maxFES<<endl;
//selectedAlg=2;
        switch( selectedAlg ){ 
            case 1:{ //1 generation of IDE
                //if(itCounter4IDE>itMax){//To doane bo bug!!!!!!!!!!!!!!!!!!!!!
                    //itMax=itCounter4IDE;
                    //cout<<"Shit"<<endl;
                //}              
                ord = order(fit.data(), popSize);
                pop = pop(ord,all).eval();
                fit = fit(ord).eval();
                CRTabUsedInEigen = CRTabUsedInEigen(ord).eval();
                newPop = pop;
                double superiorRatio = minSuperiorRatio + (1-minSuperiorRatio) * pow(10.0,(powShape*(itCounter4IDE/itMax - 1.0)));
                double pertProb = pertProbMult * superiorRatio;  //prob. of perturbation
                if( itCounter4IDE < itOfChangeSRT){
                    SRT = 0;
                }else{
                    SRT = 0.1;
                }
                for( int i=0; i<popSize; ++i){
                    selIndexes = getRandIndexesExcluding(popSize, 4, ArrayXi::Constant(1,i));
                    int oi = selIndexes(0);
                    if( itCounter4IDE <= itOfChangeSRT){
                        oi = i;
                    }
                    int r1i = selIndexes(1);
                    int r2i = selIndexes(2);
                    int r3i = selIndexes(3);
                    xo = pop(oi,all);
                    xr1= pop(r1i,all);
                    xr2= pop(r2i,all);
                    xr3= pop(r3i,all);
                    //Array<bool,Dynamic,1> ind2perturb = ArrayXd::Random(dim)+1.0 < 2.0*pertProb;
                    Array<bool,Dynamic,1> ind2perturb = Eigen::ArrayXd::NullaryExpr(dim,[&](){return random(0, 1);}) < pertProb;

                    //randIndiv = ArrayXd::Random(dim) * UP_BOUND;
                    randIndiv = Eigen::ArrayXd::NullaryExpr(dim,[&](){return random(L_BOUND, UP_BOUND);});
                    xr3 = ind2perturb.select(randIndiv, xr3);

                    double Fo = (1.0+oi) / popSize + normSigma * normRand(0,1);
                    while( Fo <= 0 || Fo > 1){
                        Fo = (1.0+oi)/popSize + normSigma * normRand(0,1);
                    }
                    int superiorSize = superiorRatio * popSize;
                    if( oi+1 > superiorSize){  //inferior
                        if( r1i+1 > superiorSize){  //  find better than superior
                            //selIndexes = getRandIndexesExcluding(superiorSize, 1, selIndexes);
    //to sposob bledny, tak jak robi Matlab !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    std::vector<size_t> indexes(superiorSize);
    iota(indexes.begin(), indexes.end(), 0);
    for(int i=0; i<selIndexes.size(); ++i){
        if( selIndexes(i) < superiorSize){
            indexes.erase( find(indexes.begin(), indexes.end(), selIndexes(i)) );
        }
    }
    selIndexes(0) = intRandom( indexes.size() ); 
//END to sposob bledny, tak jak robi Matlab!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


                            r1i = selIndexes(0);
                            xr1 =  pop(r1i, all);
                        }
                    }
                    if( (itCounter4IDE > itOfChangeSRT) && (random(0, 1) < 0.5)){  //with probability 0.1 - best point is used:
                        newPop(i, all) = pop.row(i).transpose() + Fo*(xr1 - xo) + Fo*(xr2 - xr3);

                    }else{
                        newPop(i, all) = xo + Fo*(xr1 - xo) + Fo*(xr2 - xr3);
                    }
                    //te 2 linie to to samo w wiekszości bo dla itCounter4IDE < itOfChangeSRT xo=xi a dla większych mamy xi, czasem tylko coś to random moze mieszac
                }//for
                if( random(0,1) < pEigCross){
                    int popEigSize=popSize * partOfPopUsedInEigenVect;
                    MatrixXd popEig = pop( seq(0, popEigSize-1), all );
                    MatrixXd centered = popEig.rowwise() - popEig.colwise().mean();
                    MatrixXd cov = (centered.adjoint() * centered) / double(popEig.rows() - 1);
                    SelfAdjointEigenSolver<MatrixXd> es(cov);
                    MatrixXd eigVect = es.eigenvectors();
                    for(int i=0; i<popSize;++i){
                        indAfterCross = pop(i, all);
                        mutant = newPop(i, all);
                        indEigAfterCross = eigVect.transpose() * indAfterCross.matrix();
                        mutEig = eigVect.transpose() * mutant.matrix();

                        //Array<bool,Dynamic,1> toChange = ArrayXd::Random(dim)+1.0 < 2.0*CRTabUsedInEigen(i);
                        Array<bool,Dynamic,1> toChange =  Eigen::ArrayXd::NullaryExpr(dim,[&](){return random(0, 1);})< CRTabUsedInEigen(i);
                        if( (toChange == false).all()){
                            toChange(intRandom(dim)) = true;
                        }
                        indEigAfterCross = toChange.select(mutEig, indEigAfterCross);
                        indAfterCross = (eigVect * indEigAfterCross).transpose();
                        reflection(indAfterCross, L_BOUND, UP_BOUND);
                        newPop(i, all) = indAfterCross;
                    }//for
                }else{//regular corossover
                    for(int i=0; i<popSize;++i){
                        double CR = (1.0+i)/popSize + normSigma * normRand(0,1);
                        while( CR < 0 || CR > 1){
                            CR = (1.0+i)/popSize + normSigma * normRand(0,1);
                        }
                        int jrand = intRandom(dim);
                        for( int j=0; j<dim; ++j){
                            if( !(random(0, 1) <= CR || j  == jrand)){ //use old
                                newPop(i,j) = pop(i,j);
                            }
                            //bound constr
                            if( newPop(i,j) < L_BOUND || newPop(i,j) > UP_BOUND){
                                newPop(i,j) = L_BOUND + random(0, 1)*(UP_BOUND - L_BOUND);
                            }
                        }
                    }//for
                }//end of regular crossover

                newFit = popEval( newPop);
                int numOfSucc=0;
                for(int i=0; i<popSize;++i){
                    if( newFit(i) <= fit(i) ){
                        pop.row(i) = newPop.row(i);                
                        fit(i) = newFit(i);
                        ++numOfSucc;
                    }
                }

                //indSucc = newFit <= fit;
                //int numOfSucc = indSucc.sum();
                successTab[selectedAlg-1] += numOfSucc;
                double SR = (double)numOfSucc / popSize;
                if( itCounter4IDE < itOfChangeSRT){  // stage == 1
                    if( SR <= SRT){
                        itsWithLowSRCounter = itsWithLowSRCounter + 1;
                    }else{
                        itsWithLowSRCounter = 0;
                    }
                    if( itsWithLowSRCounter >= itToEnablestage2ForIDE){    //  stage = 2;
                        itOfChangeSRT = itCounter4IDE; 
                    }
                }

                ord = order(fit.data(), popSize); //to sortowanie dla jSO, nie powinno mieć dużego znaczenia
                pop = pop(ord,all).eval();
                fit = fit(ord).eval();
                CRTabUsedInEigen = CRTabUsedInEigen(ord).eval();
                double bestErrorInIt = fit[0];
                if( bestErrorInIt < bestError){
                    bestError = bestErrorInIt;
                    bestSol = pop(0, all);
                    if( bestError <= fitError ){
                        whenFound = fitEvals;
                        break;
                    }
                }
                itCounter4IDE = itCounter4IDE + 1;
            }
                break;
                                        
            case 2:  //jSO generation
            {
                ArrayXd successCR(popSize); 
                ArrayXd successF(popSize);
                int successCounter = 0;
                //fill(deltaFit, deltaFit+popSize, -1);
                deltaFit = ArrayXd::Constant(popSize,-1);
                double perOfUsedTheBest = maxPerOfUsedTheBest - ((maxPerOfUsedTheBest - minPerOfUsedTheBest) * ((double)fitEvals / maxFES));
                bool doEigen=false;
                if(random(0, 1) < pEigCross){
                    doEigen=true;
                    ord = order(fit.data(), popSize);//sorts the rows of a matrix in ascending order
                    int popEigSize=popSize * partOfPopUsedInEigenVect;
                    MatrixXd popEig = pop( ord(seq(0, popEigSize-1)), all );
                    MatrixXd centered = popEig.rowwise() - popEig.colwise().mean();
                    MatrixXd cov = (centered.adjoint() * centered) / double(popEig.rows() - 1);
                    SelfAdjointEigenSolver<MatrixXd> es(cov);

                    eigVect = es.eigenvectors();
                }
                //MatrixXd wyn = V * D * V.inverse();
                //cout<<wyn<<endl;
                ArrayXXd newPop(popSize, dim);
                for(int i = 0; i<popSize;++i){
                    int randMemIndx = intRandom(memSize);//1 el z memSize
                    double CR = MCR[randMemIndx] + normRand(0, sqrt(normSigma));
                    CR = min( CR, 1.);
                    CR = max( CR, 0.);

                    if( fitEvals < (firstCRThreshold * maxFES)){
                        CR = max(CR, firstCRMax);
                    }else if( fitEvals < (secondCRThreshold * maxFES)){
                        CR = max(CR, secondCRMax);
                    }

                    double F = -1;
                    while( F <= 0){
                        F = random(0+tanEps, 1-tanEps) * pi - pi / 2;
                        F = tanWeight * tan(F) + MF[randMemIndx];
                    }
                    F= min(F, 1.);
                    if( (fitEvals < (boundFThreshold * maxFES)) && (F > FUpperBound) ){
                        F = FUpperBound;
                    }
                    FUsedInIt[i] = F;
                    CRUsedInIt[i] = CR;

                    int curPBestSize = max(2, (int)ceil(perOfUsedTheBest * popSize));
                    ord = order(fit.data(), popSize);//sorts the rows of a matrix in ascending order
                    ArrayXXd sortedPop = pop( ord, all );
                    int pBestIndx = intRandom(curPBestSize);
                    ArrayXd pBestSol = sortedPop(pBestIndx, all);

                    ArrayXd xi = pop(i, all);
                    double Fw;

                    ArrayXi except(1);
                    except[0]=i;
                    ArrayXi indexes = getRandIndexesExcluding(popSize, 1, except);
                    int r1Indx=indexes[0];
                    ArrayXd r1Ind = pop(r1Indx, all);
                    except.resize(2);
                    except[0]=i;
                    except[1]=r1Indx;

                    indexes = getRandIndexesExcluding(popSize + curArchSize, 1, except);
                    int r2Indx=indexes[0];
                    ArrayXd r2Ind(dim);
                    if(r2Indx<popSize){
                        r2Ind = pop(r2Indx,all);
                    }else{
                        r2Ind = archive(r2Indx-popSize, all);
                    }
                    if( fitEvals < FwFirstThreshold * maxFES){
                        Fw = FwFirstMult * F;
                    }else if( fitEvals < FwSecondThreshold * maxFES){
                        Fw = FwSecMult * F;
                    }else{
                        Fw = FwThirdMult * F;
                    }

                    ArrayXd mutant = xi + Fw * (pBestSol - xi) + F * (r1Ind - r2Ind);
                    ArrayXd indAfterCross = xi;
                    if( doEigen ){
                        VectorXd indEigAfterCross = eigVect.transpose() * indAfterCross.matrix();
                        VectorXd mutEig = eigVect.transpose() * mutant.matrix();
                        //mutEig<indEigAfterCros
                        //Array<bool,Dynamic,1> toChange = ArrayXd::Random(dim)+1.0 < 2.0*CRTabUsedInEigen(i);
                        Array<bool,Dynamic,1> toChange =  Eigen::ArrayXd::NullaryExpr(dim,[&](){return random(0, 1);})< CRTabUsedInEigen(i);
                        if( (toChange == false).all()){
                            toChange(intRandom(dim)) = true;
                        }
                        indEigAfterCross = toChange.select(mutEig, indEigAfterCross);
                        indAfterCross = (eigVect * indEigAfterCross).transpose();
                    }else{
                        //Array<bool,Dynamic,1> toChange = ArrayXd::Random(dim)+1.0 < 2.0*CR;
                        Array<bool,Dynamic,1> toChange = Eigen::ArrayXd::NullaryExpr(dim,[&](){return random(0, 1);}) < CR;
                        
                        if( (toChange == false).all()){
                            toChange(intRandom(dim)) = true;
                        }
                        indAfterCross = toChange.select( mutant, indAfterCross);
                    }
                    reflection(indAfterCross, L_BOUND, UP_BOUND);
                    newPop(i, all) = indAfterCross;
                }//for each individual
                newFit = popEval( newPop);
                for( int i = 0; i<popSize; ++i){
                    if( newFit(i) < fit(i) ){
                        deltaFit(i) = fit(i) - newFit(i);
                        successCounter = successCounter + 1;
                        if( curArchSize < maxArchSize){
                            archive(curArchSize,all) = pop(i, all);
                            ++curArchSize;
                        }else{
                            int randIndx = intRandom(curArchSize);
                            archive(randIndx, all) = pop(i, all);
                        }
                        successCR(i) =  CRUsedInIt[i];
                        successF( i) = FUsedInIt[i];
                    }
                    if( newFit(i) <= fit(i)){
                        pop(i, all) = newPop(i, all);
                        fit(i) = newFit(i);
                    }
                }//for

                if( successCounter > 0){
                    double MCR_old = MCR[curMemIndx];
                    double MF_old = MF[curMemIndx];
                    Array<bool,Dynamic,1> nonEmptyDelFitIndexes = deltaFit != -1;
                    ArrayXd nonEmptyDelFit = nonEmptyDelFitIndexes.select(deltaFit,0.0);
                    ArrayXd nonEmptySuccessCR = nonEmptyDelFitIndexes.select(successCR,0.0);
                    ArrayXd nonEmptySuccessF = nonEmptyDelFitIndexes.select(successF,0.0);
                    double sumDelta = nonEmptyDelFit.sum();
                    ArrayXd normalizedDetlas = 1.0 / sumDelta * nonEmptyDelFit;
                    double maxSCR = nonEmptySuccessCR.maxCoeff();
                    //double maxSCR = *max_element(successCR.begin(), successCR.end());
                    if( (MCR[curMemIndx] == -1) || (maxSCR == 0)){
                        MCR[curMemIndx] = -1;
                    }else{
                        ArrayXd meanSCR1 = normalizedDetlas * nonEmptySuccessCR;
                        ArrayXd meanSCR2 = meanSCR1 * nonEmptySuccessCR;
                        MCR[curMemIndx] = meanSCR2.sum()/ meanSCR1.sum();
                    }
                    ArrayXd meanSF1 = normalizedDetlas * nonEmptySuccessF;
                    ArrayXd meanSF2 = meanSF1 * nonEmptySuccessF;
                    MF[curMemIndx] = meanSF2.sum() / meanSF1.sum();
                    MCR[ curMemIndx] = (MCR[ curMemIndx] + MCR_old) / 2.0;
                    MF[ curMemIndx] = (MF[ curMemIndx] + MF_old) / 2.0;
                    ++curMemIndx;
                    if( curMemIndx >= memSize-1 ){
                        curMemIndx = 0;
                    }
                }//if( successCounter > 0){
                double fmin = *min_element(fit.begin(), fit.end());

                if( fmin < bestError){
                    bestError = fmin;
                    if( bestError <= fitError ){
                        whenFound = fitEvals;
                        break;
                    }
                }

                successTab[selectedAlg-1] += successCounter;
            }
            break;
        }//switch

        double optPopSize = round((( (double)(minPopSize - initPopSize) / maxFES) * fitEvals) + initPopSize);

        if( popSize > optPopSize){//tu w originale bylo jeszcze sortowanie populacji i CBR
            double diffPop = popSize - optPopSize;
            if( popSize - diffPop < minPopSize){
                diffPop = popSize - minPopSize;
            }

            while(diffPop>0){
                Eigen::Index worstIndx;
                fit.maxCoeff(&worstIndx);
                pop.row(worstIndx) = pop.row(--popSize);
                fit(worstIndx) = fit(popSize);
                CRTabUsedInEigen(worstIndx) = CRTabUsedInEigen(popSize);
                pop.conservativeResize(popSize, Eigen::NoChange);
                fit.conservativeResize(popSize);
                CRTabUsedInEigen.conservativeResize(popSize);
                --diffPop;
            }
//to niepotrzebne            
ord = order(fit.data(), popSize);
pop = pop(ord,all).eval();
fit = fit(ord).eval();
CRTabUsedInEigen = CRTabUsedInEigen(ord).eval();            
//end to niepotrzebne               
            maxArchSize = round(popSize * archSizeMult);
            while( curArchSize > maxArchSize){
                int indxToDel = intRandom(curArchSize);  
                //archive.row(indxToDel)=archive.row(--curArchSize);
                for(int i=indxToDel; i<curArchSize-1; ++i){
                    archive.row(i)=archive.row(i+1);
                }
                --curArchSize;
            }
        }//if( popSize > optPopSize){
    }//while(main loop)    
    coco_suite_free(suite);
    delete [] FUsedInIt;
    delete [] CRUsedInIt;
}

