#include <iostream>
#include <fstream>
#include <random>
#include <vector>

#include <boost/program_options.hpp>

#include "utils.h"
#include "optimizer.h"


namespace po = boost::program_options;
extern std::mt19937 generator;

using namespace std;
using Eigen::all;

//tuned4CEC ./a.out --dim 10 --seed 1234 --res_file_name jSO_IDE_4CEC --algProbResetThreshold 0.09 --initialSuccessCount 27 --pop_size 85 --norm_sigma 0.28 --partOfPopUsedInEigenVect 0.74 --pEigCross 0.21 --smallCauchyThreshold 0.35 --arch_size_mult 2.06 --maxPerOfUsedTheBest 0.35 --tanWeight 0.19 --FwFirstMult 0.87 --FwSecMult 0.84 --FwFirstThreshold 0.34 --FwSecondThreshold 0.56 --pertProbMult 0.08 --minSuperiorRatio 0.11 --powShape 5.92
//default ./a.out --dim 10 --seed 1234 --res_file_name default --pop_size 100 --initial_CR 0.8 --initial_F 0.3 --norm_sigma 0.1 --cauchy_sigma 0.1
//debug ./a.out --seed 1234 --res_file_name default --pop_size 12

int main(int argc, char* argv[]){
    po::options_description desc("Allowed options");  
    unsigned int seed;
    std::string resFileName;

    //common parameters:
    int popSize=100;
    float norm_sigma=0.1;
    int min_pop_size=10;
    //eig
    double pEigCross = 0.4;
    double partOfPopUsedInEigenVect = 0.5;
    float cauchy_sigma=0.1;
    double smallCauchyThreshold=0.5;

    //IDE
    int itToEnableStage2ForIDEDivider=10;
    double itOfChangeSRTDivider=2;
    double powShape=5.0; //1..6
    double minSuperiorRatio=0.1; //0.05 - 0.2
    double pertProbMult=0.1;
    double SRT = 0.1;

    //jSo
    float initialCR=0.8;
    float initialF=0.3;
    int memSize = 5;    
    double arch_size_mult=2.6;
    double maxPerOfUsedTheBest = 0.25;
    double firstCRThreshold=0.25;
    double secondCRThreshold=0.5;
    double firstCRMax=0.7;
    double secondCRMax=0.6;    
    double tanEps=0;
    double tanWeight=0.1;
    double FUpperBound=0.7;    
    double boundFThreshold=0.6;
    double FwFirstMult=0.7;
    double FwSecMult=0.8;
    double FwThirdMult=1.2;
    double FwFirstThreshold=0.2;
    double FwSecondThreshold=0.4;

    //EA4Eig
    double algProbResetThreshold=0.1;
    int initialSuccessCount=2;

size_t dim=10;

    desc.add_options ()
        ("help", "print usage message")
        ("dim", po::value(&dim)->required(), "dim")
        ("seed", po::value(&seed)->required(), "seed")
        ("pop_size", po::value( &popSize )->required(), "pop_size")
        ("initial_CR", po::value( &initialCR ), "initial_CR")
        ("initial_F", po::value( &initialF ), "initial_F")
        ("res_file_name", po::value<std::string>( &resFileName )->required(), "result file base name")
        ("norm_sigma", po::value(&norm_sigma), "norm_sigma")
        ("cauchy_sigma", po::value(&cauchy_sigma), "cauchy_sigma")
        ("algProbResetThreshold", po::value(&algProbResetThreshold), "algProbResetThreshold")
        ("initialSuccessCount", po::value(&initialSuccessCount), "initialSuccessCount")
        ("min_pop_size", po::value(&min_pop_size), "min_pop_size")
        ("itToEnableStage2ForIDEDivider", po::value(&itToEnableStage2ForIDEDivider), "itToEnableStage2ForIDEDivider")
        ("itOfChangeSRTDivider", po::value(&itOfChangeSRTDivider), "itOfChangeSRTDivider")
        ("partOfPopUsedInEigenVect", po::value(&partOfPopUsedInEigenVect), "partOfPopUsedInEigenVect")
        ("pEigCross", po::value(&pEigCross), "pEigCross")
        ("smallCauchyThreshold ", po::value(&smallCauchyThreshold ), "smallCauchyThreshold=0.5 ")
        ("arch_size_mult", po::value(&arch_size_mult), "archive size multiplier")
        ("memSize", po::value(&memSize), "memSize")
        ("maxPerOfUsedTheBest", po::value(&maxPerOfUsedTheBest), "maxPerOfUsedTheBest")
        ("powShape", po::value(&powShape), "powShape")
        ("minSuperiorRatio", po::value(&minSuperiorRatio), "minSuperiorRatio")
        ("pertProbMult", po::value(&pertProbMult), "pertProbMult")
        ("SRT", po::value(&SRT), "SRT")
        ("firstCRThreshold", po::value(&firstCRThreshold), "firstCRThreshold")
        ("secondCRThreshold", po::value(&secondCRThreshold), "secondCRThreshold")
        ("firstCRMax", po::value(&firstCRMax), "firstCRMax")
        ("secondCRMax", po::value(&secondCRMax), "secondCRMax")
        ("tanEps", po::value(&tanEps), "tanEps")
        ("tanWeight", po::value(&tanWeight), "tanWeight")
        ("FUpperBound", po::value(&FUpperBound), "FUpperBound")
        ("boundFThreshold", po::value(&boundFThreshold), "boundFThreshold")
        ("FwFirstMult", po::value(&FwFirstMult), "FwFirstMult")
        ("FwSecMult", po::value(&FwSecMult), "FwSecMult")
        ("FwThirdMult", po::value(&FwThirdMult), "FwThirdMult")
        ("FwFirstThreshold", po::value(&FwFirstThreshold), "FwFirstThreshold")
        ("FwSecondThreshold", po::value(&FwSecondThreshold), "FwSecondThreshold")
        ; 
        
    const int MAX_FUN_NMBR=12;
    po::variables_map vm;
    po::store (po::command_line_parser (argc, argv).options (desc).run (), vm);
    if(vm.count("help")){
        std::cout << desc << "\n";
        return 0;
    }
    po::notify(vm);

    generator.seed(seed);
    double avResToRet=0;
    int maxFES = 10000*dim;
    if(dim==10){
        maxFES = 200000;
    }else{
        maxFES = 1000000;
    }
    int NMBR_OF_RUNS=30;
    double FIT_ERROR=1e-8;
    double fitAtOptTab[] = {300, 400, 600, 800, 900, 1800, 2000, 2200, 2300, 2400, 2600, 2700};
    
    const int NUM_OF_DUMPS=16;
    size_t evalsToDumpSize = NUM_OF_DUMPS;
    std::vector<size_t> evalsToDump(evalsToDumpSize);
    for(size_t i=0; i<evalsToDumpSize; ++i){
        evalsToDump[i] = pow(double(dim),double(i)/5.0-3.0)*maxFES;
    }
    evalsToDump[NUM_OF_DUMPS-1] = maxFES;

    for (int funNmbr = 1; funNmbr <= MAX_FUN_NMBR; ++funNmbr){
        ArrayXXd resTab(NMBR_OF_RUNS, evalsToDumpSize+1);
        double fitAtOpt = fitAtOptTab[funNmbr-1];
        for (int runNum = 0; runNum != NMBR_OF_RUNS; ++runNum){
            cout<< "dim:"<<dim<<", fun:"<<funNmbr<<", run:"<<runNum+1<<endl;            
            Optimizer optimizer(dim, funNmbr, maxFES, fitAtOpt, FIT_ERROR, popSize, initialCR, initialF, evalsToDump, 
                norm_sigma, cauchy_sigma, algProbResetThreshold, initialSuccessCount,min_pop_size, itToEnableStage2ForIDEDivider,
                itOfChangeSRTDivider, partOfPopUsedInEigenVect, pEigCross, smallCauchyThreshold, arch_size_mult,
                memSize,maxPerOfUsedTheBest, powShape, minSuperiorRatio, pertProbMult, SRT,firstCRThreshold,
                secondCRThreshold, firstCRMax, secondCRMax, tanEps, tanWeight, FUpperBound, boundFThreshold,
                FwFirstMult, FwSecMult, FwThirdMult, FwFirstThreshold, FwSecondThreshold);
                optimizer.optimize();
            resTab(runNum, Eigen::seqN(0,evalsToDumpSize)) = optimizer.getResDump();
            resTab(runNum, evalsToDumpSize) = optimizer.whenFound;
        }//foreach run
        string curResFileName = resFileName+"_" + to_string(funNmbr)+"_"+to_string(dim)+ ".txt";
        ofstream resArrayFile( curResFileName.c_str(), ios::trunc );
        for(int step = 0; step!=evalsToDumpSize+1; ++step){
            for( int runNum = 0; runNum!=NMBR_OF_RUNS; ++runNum ){
                resArrayFile<<resTab(runNum,step)<<"\t";
            }
            resArrayFile<<endl;
        }
    }//foreach fun
    return 0;
}
