#pragma once
#include <vector>
#include <eigen3/Eigen/Dense>
 
using Eigen::MatrixXd;
using Eigen::VectorXd;

class Optimizer{
public:
    Optimizer( int dim, int funNmbr, int maxFES, double fitAtOpt, double fitError, int popSize, double initialCR, double initialF,
        std::vector<size_t> evalsToDump, double normSigma, double cauchySigma, double algProbResetThreshold, int initialSuccessCount,
        int min_pop_size, int itToEnableStage2ForIDEDivider, double itOfChangeSRTDivider, double partOfPopUsedInEigenVect,
        double pEigCross, double smallCauchyThreshold, double arch_size_mult, int memSize, double maxPerOfUsedTheBest,
        double powShape, double minSuperiorRatio, double pertProbMult, double SRT, double firstCRThreshold,
        double secondCRThreshold, double firstCRMax, double secondCRMax, double tanEps, double tanWeight, double FUpperBound,
        double boundFThreshold, double FwFirstMult, double FwSecMult, double FwThirdMult, double FwFirstThreshold,
        double FwSecondThreshold);
    ~Optimizer();
    void optimize();
    double getBestError();
    bool budgetLeft();
    ArrayXd getResDump();
    int whenFound;
private:
    double callObjFunc(double* sol);
    ArrayXd popEval(const ArrayXXd& pop);
    void rouleteAlgSel(int* successTab, int nmbrOfAlgs, int& selectedAlg, double& minProbOfAlgSel);
    void saveInitialFitDump();
    int dim;
    int funNmbr;
    int maxFES;

    double fitAtOpt;
    double fitError;
    int initPopSize;
    int popSize;
    int minPopSize;
    double initialCR;
    double initialF;
    int fitEvals;
    double bestError;
    double bestErrorAtEv;
    ArrayXXd pop;
    ArrayXd fit;
    ArrayXd bestSol;
    std::vector<size_t> evalsToDump;
    size_t curDumpIndx;
    ArrayXd resultsDumps;
    double normSigma;
    double cauchySigma;
    double algProbResetThreshold;
    int initialSuccessCount;
    int itToEnableStage2ForIDEDivider;
    double itOfChangeSRTDivider;
    double partOfPopUsedInEigenVect;
    double pEigCross;
    double smallCauchyThreshold;
    double archSizeMult;
    int memSize;
    double maxPerOfUsedTheBest;
    double powShape;
    double minSuperiorRatio;
    double pertProbMult;
    double SRT;
    double firstCRThreshold;
    double secondCRThreshold;
    double firstCRMax;
    double secondCRMax;
    double tanEps;
    double tanWeight;
    double FUpperBound;
    double boundFThreshold;
    double FwFirstMult;
    double FwSecMult;
    double FwThirdMult;
    double FwFirstThreshold;
    double FwSecondThreshold;
};

