#include <random>
#include <algorithm>

#include "utils.h"

using Eigen::Map;

std::mt19937 generator;
std::uniform_int_distribution<int> uni_int(0,32768);
std::uniform_real_distribution<double> uni_real(0.0, 1.0);
std::normal_distribution<double> norm_dist(0.0, 1.0);
std::cauchy_distribution<double> cauchy_dist(0.0, 1.0);

int intRandom(int maxPl1){
    if(maxPl1 == 0)
        return 0;
    return uni_int(generator)%maxPl1;
}

double random( double minimal, double maximal ){
    return uni_real(generator)*(maximal-minimal)+minimal;
}

double normRand( double mu, double sigma ){
    return norm_dist(generator)*sigma + mu;
}

double cauchyRand( double mu, double sigma ){
    return cauchy_dist(generator)*sigma+mu;
}

ArrayXi order(double* array, int size){
    std::vector<int> indices(size);
    // iota is from <numeric>, C++0x
    std::iota(indices.begin(), indices.end(), 0);

    std::sort(indices.begin(), indices.end(), [array](std::size_t left, std::size_t right){
        return array[left] < array[right];
    });
    Map<ArrayXi> v1(indices.data(),size);
    return v1;
}

ArrayXi getRandIndexesExcluding(int popSize, int numOfIndexes, const ArrayXi& except){
    std::vector<size_t> indexes(popSize);
    iota(indexes.begin(), indexes.end(), 0);
    for(int i=0; i<except.size(); ++i){
        if( except(i) < popSize){
            indexes.erase( find(indexes.begin(), indexes.end(), except(i)) );
        }
    }
    ArrayXi toRet(numOfIndexes);
    for(int i=0; i<numOfIndexes; ++i){
        if(indexes.size()>0){
            toRet(i) = indexes[ intRandom( indexes.size() ) ];
            indexes.erase( find(indexes.begin(), indexes.end(), toRet(i) ) );
        }else{
            toRet(i)=0;
        }
    }
    return toRet;
}

void reflection(ArrayXd& sol, double lower, double upper){
    for(int j=0; j<sol.size(); ++j){
        while( sol(j)<lower || sol(j)>upper ){
            if( sol(j) > upper ){
                sol(j) = 2*upper-sol(j);
            }else if( sol(j) < lower ){
                sol(j) = 2*lower-sol(j);
            }
        }//while
    }//for
}

