#include <algorithm>
#include <numeric>
#include <iostream>
#include <limits>
#include <random>
#include <vector>

#include "utils.h"
#include "optimizer.h"

void cec22_test_func(double *, double *,int,int,int);

using namespace std;
using Eigen::all;
using Eigen::seq;
using Eigen::seqN;
using Eigen::SelfAdjointEigenSolver;
using Eigen::Array;
using Eigen::Dynamic;

const double pi=3.14159265358979323846;

extern std::mt19937 generator;

ArrayXd Optimizer::getResDump(){
    return resultsDumps;
}

Optimizer::Optimizer( int dim, int funNmbr, int maxFES, double fitAtOpt, double fitError, int popSize, double initial_CR, double initial_F,
    std::vector<size_t> evalsToDump, double normSigma, double cauchySigma, double algProbResetThreshold, int initialSuccessCount, 
    int min_pop_size, int failsThDiv, double tMaxDiv, double partOfPopUsedInEigenVect,
    double pEigCross, double smallCauchyThreshold, double arch_size_mult, int memSize, double maxPerOfUsedTheBest,
    double powShape, double minSuperiorRatio, double pertProbMult, double SRT, double firstCRThreshold,
    double secondCRThreshold, double firstCRMax, double secondCRMax, double tanEps, double tanWeight,
    double FUpperBound, double boundFThreshold, double FwFirstMult, double FwSecMult, double FwThirdMult, 
    double FwFirstThreshold, double FwSecondThreshold):
    dim(dim), funNmbr(funNmbr), maxFES(maxFES), fitAtOpt(fitAtOpt), fitError(fitError), initPopSize(popSize), initialCR(initial_CR), initialF(initial_F),
    fitEvals(0), bestError(numeric_limits<double>::infinity()), bestSol(dim), evalsToDump(evalsToDump), normSigma(normSigma),
    cauchySigma(cauchySigma), whenFound(maxFES), algProbResetThreshold(algProbResetThreshold), initialSuccessCount(initialSuccessCount),
    minPopSize(min_pop_size), failsThDiv(failsThDiv), tMaxDiv(tMaxDiv),
    partOfPopUsedInEigenVect(partOfPopUsedInEigenVect), pEigCross(pEigCross), smallCauchyThreshold(smallCauchyThreshold),
    archSizeMult(arch_size_mult), memSize(memSize), maxPerOfUsedTheBest(maxPerOfUsedTheBest), powShape(powShape),
    minSuperiorRatio(minSuperiorRatio), pertProbMult(pertProbMult), SRT(SRT), firstCRThreshold(firstCRThreshold),
    secondCRThreshold(secondCRThreshold), firstCRMax(firstCRMax), secondCRMax(secondCRMax), tanEps(tanEps), tanWeight(tanWeight),
    FUpperBound(FUpperBound),boundFThreshold(boundFThreshold), FwFirstMult(FwFirstMult), FwSecMult(FwSecMult),
    FwThirdMult(FwThirdMult), FwFirstThreshold(FwFirstThreshold), FwSecondThreshold(FwSecondThreshold){
}

Optimizer::~Optimizer(){
}

double Optimizer::callObjFunc(double* sol){
    double toRet;
    cec22_test_func(sol, &toRet, dim, 1, funNmbr);
    toRet -= fitAtOpt;
    ++fitEvals;
    if(toRet<bestErrorAtEv){
       bestErrorAtEv = toRet;
    }
    if(fitEvals==evalsToDump[curDumpIndx]){
        resultsDumps[curDumpIndx++] = bestErrorAtEv;
    }
    return toRet;
}

void Optimizer::saveInitialFitDump(){
    while(evalsToDump[curDumpIndx]<=fitEvals ){
        resultsDumps[curDumpIndx++] = bestErrorAtEv;
    }
}

double Optimizer::getBestError(){
    return bestError;
}

bool Optimizer::budgetLeft(){
    return fitEvals<maxFES;
}

ArrayXd Optimizer::popEval(const ArrayXXd& pop){
    ArrayXd fitToRet(mu);
    for(int i=0; i<mu; ++i){
        ArrayXd vectorXd = pop.row(i);
        fitToRet(i) = callObjFunc(vectorXd.data());
    }
    return fitToRet;
}

void Optimizer::optimize(){
    resultsDumps.setZero( evalsToDump.size() );
    curDumpIndx=0;
    bestErrorAtEv=numeric_limits<double>::infinity();
    const int L_BOUND=-100;
    const int UP_BOUND=100;
    mu = initPopSize;
    if(minPopSize>initPopSize){
        minPopSize=initPopSize;
    }
    pop = Eigen::ArrayXXd::NullaryExpr(mu, dim,[&](){return random(L_BOUND, UP_BOUND);});
    fit.setConstant(mu, numeric_limits<double>::infinity());

    fit = popEval(pop);
    saveInitialFitDump();
    auto it = min_element(fit.begin(), fit.end());
    bestError = *it;
    bestSol = pop.row(distance(fit.begin(), it));

    int tMax = maxFES / mu;
    int tTh = tMax / tMaxDiv; // when switch to the second stage
    int t = 0;


    ArrayXd CRTabUsedInEigen = ArrayXd::Zero(mu);
    for(int i=0; i<mu; ++i){
        if( random(0, 1) < smallCauchyThreshold ){
            CRTabUsedInEigen(i) = cauchyRand(0.1, cauchySigma);
        }else{
            CRTabUsedInEigen(i) = cauchyRand(0.95, cauchySigma);
        }
        if( CRTabUsedInEigen(i) > 1){
            CRTabUsedInEigen(i) = 1;
        }else if(CRTabUsedInEigen(i) < 0){
            CRTabUsedInEigen(i) = 0;
        }
    }

    ArrayXi ord(mu);
    ArrayXXd newPop(mu, dim);
    ArrayXi selIndexes;
    ArrayXd xo, xr1, xr2, xr3;
    ArrayXd indAfterCross;
    ArrayXd mutant;
    VectorXd indEigAfterCross;
    VectorXd mutEig;
    ArrayXd newFit;
    MatrixXd eigVect;
    while( (fitEvals < maxFES) && (bestError > fitError)){
{ //1 generation of IDE
                if(t>tMax){
                    tMax=t;
                }            
                ord = order(fit.data(), mu);
                pop = pop(ord,all).eval();
                fit = fit(ord).eval();
                CRTabUsedInEigen = CRTabUsedInEigen(ord).eval();
                newPop = pop;
                double superiorRatio = minSuperiorRatio + (1-minSuperiorRatio) * pow(10.0,(powShape*(t/tMax - 1.0)));
                double pertProb = pertProbMult * superiorRatio;  //prob. of perturbation
                for( int i=0; i<mu; ++i){
                    selIndexes = getRandIndexesExcluding(mu, 4, ArrayXi::Constant(1,i));
                    int oi = selIndexes(0);
                    if( t <= tTh){
                        oi = i;
                    }
                    int r1i = selIndexes(1);
                    int r2i = selIndexes(2);
                    int r3i = selIndexes(3);
                    xo = pop(oi,all);
                    xr1= pop(r1i,all);
                    xr2= pop(r2i,all);
                    xr3= pop(r3i,all);

                    double Fo = (1.0+oi) / mu + normSigma * normRand(0,1);
                    while( Fo <= 0 || Fo > 1){
                        Fo = (1.0+oi)/mu + normSigma * normRand(0,1);
                    }
                    int superiorSize = superiorRatio * mu;
                    if( oi+1 > superiorSize){  //inferior
                        if( r1i+1 > superiorSize){  //  find better than superior
                            selIndexes = getRandIndexesExcluding(superiorSize, 1, selIndexes);
                            r1i = selIndexes(0);
                            xr1 =  pop(r1i, all);
                        }
                    }
                    newPop(i, all) = xo + Fo*(xr1 - xo) + Fo*(xr2 - xr3);
                }//for
                if( random(0,1) < pEigCross){
                    int popEigSize=mu * partOfPopUsedInEigenVect;
                    MatrixXd popEig = pop( seq(0, popEigSize-1), all );
                    MatrixXd centered = popEig.rowwise() - popEig.colwise().mean();
                    MatrixXd cov = (centered.adjoint() * centered) / double(popEig.rows() - 1);
                    SelfAdjointEigenSolver<MatrixXd> es(cov);
                    MatrixXd eigVect = es.eigenvectors();
                    for(int i=0; i<mu;++i){
                        indAfterCross = pop(i, all);
                        mutant = newPop(i, all);
                        indEigAfterCross = eigVect.transpose() * indAfterCross.matrix();
                        mutEig = eigVect.transpose() * mutant.matrix();

                        Array<bool,Dynamic,1> toChange =  Eigen::ArrayXd::NullaryExpr(dim,[&](){return random(0, 1);})< CRTabUsedInEigen(i);
                        if( (toChange == false).all()){
                            toChange(intRandom(dim)) = true;
                        }
                        indEigAfterCross = toChange.select(mutEig, indEigAfterCross);
                        indAfterCross = (eigVect * indEigAfterCross).transpose();
                        reflection(indAfterCross, L_BOUND, UP_BOUND);
                        newPop(i, all) = indAfterCross;
                    }//for
                }else{//regular corossover
                    for(int i=0; i<mu;++i){
                        double CR = (1.0+i)/mu + normSigma * normRand(0,1);
                        while( CR < 0 || CR > 1){
                            CR = (1.0+i)/mu + normSigma * normRand(0,1);
                        }
                        int jrand = intRandom(dim);
                        for( int j=0; j<dim; ++j){
                            if( !(random(0, 1) <= CR || j  == jrand)){ //use old
                                newPop(i,j) = pop(i,j);
                            }
                            //bound constr
                            if( newPop(i,j) < L_BOUND || newPop(i,j) > UP_BOUND){
                                newPop(i,j) = L_BOUND + random(0, 1)*(UP_BOUND - L_BOUND);
                            }
                        }
                    }//for
                }//end of regular crossover

                newFit = popEval( newPop);
                for(int i=0; i<mu;++i){
                    if( newFit(i) <= fit(i) ){
                        pop.row(i) = newPop.row(i);                
                        fit(i) = newFit(i);
                    }
                }

                double bestErrorInIt = fit.minCoeff();
                if( bestErrorInIt < bestError){
                    bestError = bestErrorInIt;
                    ord = order(fit.data(), mu);
                    bestSol = pop(ord,all).row(0);
                    if( bestError <= fitError ){
                        whenFound = fitEvals;
                        break;
                    }
                }
                t = t + 1;
            }
                                        

        double optPopSize = round((( (double)(minPopSize - initPopSize) / maxFES) * fitEvals) + initPopSize);

        if( mu > optPopSize){
            double diffPop = mu - optPopSize;
            if( mu - diffPop < minPopSize){
                diffPop = mu - minPopSize;
            }

            while(diffPop>0){
                Eigen::Index worstIndx;
                fit.maxCoeff(&worstIndx);
                pop.row(worstIndx) = pop.row(--mu);
                fit(worstIndx) = fit(mu);
                CRTabUsedInEigen(worstIndx) = CRTabUsedInEigen(mu);
                pop.conservativeResize(mu, Eigen::NoChange);
                fit.conservativeResize(mu);
                CRTabUsedInEigen.conservativeResize(mu);
                --diffPop;
            }             
        }//if( mu > optPopSize){
    }//while(main loop)    
}

